{% extends "admin/change_list.html" %}
{% load i18n admin_urls admin_static admin_list custom_admin %}
{% block result_list %}
  {% if action_form and actions_on_top %}{% custom_admin_actions %}{% endif %}
  {% result_list cl %}
  {% if action_form and actions_on_bottom %}{% custom_admin_actions %}{% endif %}
{% endblock %}
